package apachetest;

import java.util.List;
import java.util.Map;

import org.jboss.netty.buffer.ChannelBuffers;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelFuture;
import org.jboss.netty.channel.ChannelFutureListener;
import org.jboss.netty.channel.ChannelPipelineCoverage;
import org.jboss.netty.handler.codec.http.DefaultHttpResponse;
import org.jboss.netty.handler.codec.http.HttpHeaders;
import org.jboss.netty.handler.codec.http.HttpRequest;
import org.jboss.netty.handler.codec.http.HttpResponse;
import org.jboss.netty.handler.codec.http.HttpResponseStatus;
import org.jboss.netty.handler.codec.http.HttpVersion;
import org.jboss.netty.handler.codec.http.QueryStringDecoder;

@ChannelPipelineCoverage("one")
public class SimpleHttpHandler extends AbstractHttpHandler {

	private String serverInstanceId;
	
	private String sessionInstanceSeparator;
	
	private static ThreadLocal<StringBuffer> sessionIdBufLocal = new ThreadLocal<StringBuffer>(){

		@Override
		public StringBuffer get() {
			StringBuffer strBuf=super.get();
			strBuf.setLength(0);
			return strBuf;
		}

		@Override
		protected StringBuffer initialValue() {
			return new StringBuffer(50);
		}
		
	};
	
	public SimpleHttpHandler(String serverInstanceId, String sessionInstanceSeparator) {
		this.serverInstanceId = serverInstanceId;
		this.sessionInstanceSeparator = sessionInstanceSeparator;
	}
	
	
	
	protected void handleRequest(HttpRequest request, Channel userChannel) throws Exception {
		 String requestUri = request.getUri();
		 QueryStringDecoder queryStringDecoder = new QueryStringDecoder(requestUri);
		 Map<String, List<String>> params = queryStringDecoder.getParameters();
		 String sessionId = null;
		int indexOf = requestUri.indexOf("sessionTest");
		if (indexOf==-1) {
			sendResponse(request, userChannel, 
					HttpResponseStatus.NOT_FOUND, "Valid URI supported is /sessionTest");
			return;
		}
		indexOf = requestUri.indexOf(";jsessionid=");
		if (indexOf>-1) {
			int endIndex = requestUri.indexOf('?');
			if (endIndex<indexOf) {
				endIndex = -1;
			}
			int beginIndex = indexOf+";jsessionid=".length();
			if (endIndex>beginIndex) {
				sessionId = requestUri.substring(beginIndex, endIndex);
			} else 
				sessionId = requestUri.substring(beginIndex);
		}
		List<String> sessionList = params.get("jsessionid");
		if (sessionList!=null && !sessionList.isEmpty()) {
			String sessionInList = sessionList.get(0);
			if (sessionInList!=null && !"".equals(sessionInList.trim())) {
				sessionId = sessionInList;
			}
		}
		List<String> userList = params.get("user");
		String user = null;
		if (userList!=null && !userList.isEmpty()) {
			user = userList.get(0);
		}
		if (sessionId==null) {
			if (user==null || "".equals(user)) {
				sendResponse(request, userChannel, 
						HttpResponseStatus.UNUATHORIZED, "No valid session, please login with user");
				return;
			}
			UserSessionDetails userSessionDetails = SessionIdHolder.userIdToUserMap.get(user);
			if (userSessionDetails==null) {
				userSessionDetails = new UserSessionDetails();
				userSessionDetails.userChannel = userChannel;
				userSessionDetails.userId = user;
				SessionIdHolder.userIdToUserMap.putIfAbsent(user, userSessionDetails);
				userSessionDetails = SessionIdHolder.userIdToUserMap.get(user);
			}
			synchronized(userSessionDetails) {
				if (userSessionDetails.sessionId!=null) {
					SessionIdHolder.sessionIdToUserMap.remove(userSessionDetails.sessionId);
				}
				sessionId = userSessionDetails.sessionId = IdGenerator.generateUniqueId();
				userSessionDetails.sessionCreationTime = System.currentTimeMillis();
				userSessionDetails.lastAccessTime = System.currentTimeMillis();
				SessionIdHolder.sessionIdToUserMap.put(sessionId, userSessionDetails);
				SessionIdHolder.userIdToUserMap.put(user, userSessionDetails);
			}
			
			StringBuffer sessionIdToReturn = sessionIdBufLocal.get();
			sessionIdToReturn.append(sessionId);
			if (serverInstanceId!=null && !"".equals(serverInstanceId)) {
				sessionIdToReturn.append(sessionInstanceSeparator).append(serverInstanceId);
			}
			sendResponse(request,userChannel,HttpResponseStatus.OK, "Session Id is "+sessionIdToReturn);
			
		} else {
			//strip off the identity of the server
			int serverIdIndex = sessionId.indexOf(sessionInstanceSeparator);
			if (serverIdIndex>=0)
				sessionId = sessionId.substring(0, serverIdIndex);
			UserSessionDetails userSessionDetails = SessionIdHolder.sessionIdToUserMap.get(sessionId);
			if (userSessionDetails==null) {
				sendResponse(request, userChannel, 
						HttpResponseStatus.UNUATHORIZED, "No valid session, please login with user");
				return;
			}
			synchronized(userSessionDetails) {
				userSessionDetails.userChannel = userChannel;
				userSessionDetails.lastAccessTime = System.currentTimeMillis();
				UserSessionDetails userDetails = SessionIdHolder.userIdToUserMap.get(userSessionDetails.userId);
				if (userDetails==null || userDetails!=userSessionDetails) {
					SessionIdHolder.userIdToUserMap.put(userSessionDetails.userId, userSessionDetails);
				}
			}
			sendResponse(request,userChannel,HttpResponseStatus.OK, "Welcome user "+userSessionDetails.userId+" to server "+serverInstanceId);
		}
		
	}
	
	private void sendResponse(HttpRequest request, Channel userChannel, 
			HttpResponseStatus responseStatus, String responseBody) throws Exception {
		boolean close =
            HttpHeaders.Values.CLOSE.equalsIgnoreCase(request.getHeader(HttpHeaders.Names.CONNECTION)) ||
            request.getProtocolVersion().equals(HttpVersion.HTTP_1_0) &&
            !HttpHeaders.Values.KEEP_ALIVE.equalsIgnoreCase(request.getHeader(HttpHeaders.Names.CONNECTION));
		HttpResponse response = new DefaultHttpResponse(HttpVersion.HTTP_1_1, responseStatus);
		response.setHeader(HttpHeaders.Names.CACHE_CONTROL, "no-store");
		response.setHeader(HttpHeaders.Names.PRAGMA, "no-cache");
		response.setHeader(HttpHeaders.Names.CONTENT_LENGTH, ""+((responseBody==null)?0:responseBody.length()));
		byte[] array = responseBody.getBytes("UTF8");
		if (responseBody!=null)
			response.setContent(ChannelBuffers.wrappedBuffer(array));
		ChannelFuture future = userChannel.write(response);
		if (close) {
            future.addListener(ChannelFutureListener.CLOSE);
        }
	}

	

	
}
